@@ -18,7 +18,11 @@ import socket
 import logging
 
 from apscheduler.schedulers.background import BackgroundScheduler
-from flask_apscheduler.utils import import_string
+from apscheduler.util import obj_to_ref
+from apscheduler.util import ref_to_obj
+from flask_apscheduler.views import get_job
+from flask_apscheduler.views import get_jobs
+from flask_apscheduler.views import run_job
 
 LOGGER = logging.getLogger(__name__)
 
@@ -50,6 +54,32 @@ class APScheduler(object):
         if not app:
             raise ValueError('app must be a Flask application')
 
+        app.apscheduler = self
+
+        self.__load_config(app)
+        self.__load_views(app)
+
+    def get_job(self, job_id):
+        return self.__scheduler.get_job(job_id)
+
+    def get_jobs(self):
+        return self.__scheduler.get_jobs()
+
+    def start(self):
+        if not self.allowed_hosts:
+            LOGGER.debug('None server allowed to start the scheduler.')
+
+        if self.host_name not in self.allowed_hosts and '*' not in self.allowed_hosts:
+            LOGGER.debug('Host name %s is not allowed to start the APScheduler. Servers allowed: %s' %
+                         (self.host_name, ','.join(self.allowed_hosts)))
+            return
+
+        self.__scheduler.start()
+
+    def shutdown(self, wait=True):
+        self.__scheduler.shutdown(wait)
+
+    def __load_config(self, app):
         options = dict()
 
         job_stores = app.config.get('APSCHEDULER_JOBSTORES')
@@ -72,80 +102,69 @@ class APScheduler(object):
 
         jobs = app.config.get('APSCHEDULER_JOBS')
 
-        if isinstance(jobs, list):
-            for job in jobs:
-                self.__load_job(app, job)
-
-        if isinstance(jobs, dict):
-            for id, job in jobs.items():
-                self.__load_job(app, job, id)
+        for job in jobs:
+            self.__load_job(app, job)
 
         hosts = app.config.get('APSCHEDULER_ALLOWED_HOSTS')
         if hosts:
             for host in hosts:
                 self.__allowed_hosts.append(host.lower())
 
-    def start(self):
-        if not self.allowed_hosts:
-            LOGGER.debug('None server allowed to start the scheduler.')
-
-        if self.host_name not in self.allowed_hosts and '*' not in self.allowed_hosts:
-            LOGGER.debug('Host name %s is not allowed to start the APScheduler. Servers allowed: %s' %
-                         (self.host_name, ','.join(self.allowed_hosts)))
-            return
-
-        self.__scheduler.start()
-
-    def shutdown(self, wait=True):
-        self.__scheduler.shutdown(wait)
-
-    def __load_job(self, app, job, id=None):
+    def __load_job(self, app, job):
         def call_func(*args, **kwargs):
             with app.app_context():
                 func(*args, **kwargs)
 
         func = job.get('func')
-
-        if not func:
-            raise Exception('func is mandatory.')
+        func_ref = None
 
         if isinstance(func, str):
-            func = import_string(func)
+            func = ref_to_obj(func)
+            func_ref = func
 
-        if not hasattr(func, '__call__'):
-            raise Exception('func must be a function.')
+        if callable(func):
+            func_ref = obj_to_ref(func)
 
-        trigger = job.get('trigger', 'date')
+        id = job.get('id')
+        trigger = job.get('trigger')
         func_args = job.get('args')
         func_kwargs = job.get('kwargs')
 
         trigger_args = dict()
+        trigger_type = trigger.get('type')
 
-        if trigger == 'interval':
-            self.__copy_item('weeks', job, trigger_args)
-            self.__copy_item('days', job, trigger_args)
-            self.__copy_item('hours', job, trigger_args)
-            self.__copy_item('minutes', job, trigger_args)
-            self.__copy_item('seconds', job, trigger_args)
-            self.__copy_item('start_date', job, trigger_args)
-            self.__copy_item('end_date', job, trigger_args)
-            self.__copy_item('timezone', job, trigger_args)
-        elif trigger == 'cron':
-            self.__copy_item('year', job, trigger_args)
-            self.__copy_item('month', job, trigger_args)
-            self.__copy_item('day', job, trigger_args)
-            self.__copy_item('week', job, trigger_args)
-            self.__copy_item('day_of_week', job, trigger_args)
-            self.__copy_item('hour', job, trigger_args)
-            self.__copy_item('minute', job, trigger_args)
-            self.__copy_item('second', job, trigger_args)
-        elif trigger == 'date':
-            self.__copy_item('run_date', job, trigger_args)
-            self.__copy_item('timezone', job, trigger_args)
+        if trigger_type == 'interval':
+            self.__copy_item('weeks', trigger, trigger_args)
+            self.__copy_item('days', trigger, trigger_args)
+            self.__copy_item('hours', trigger, trigger_args)
+            self.__copy_item('minutes', trigger, trigger_args)
+            self.__copy_item('seconds', trigger, trigger_args)
+            self.__copy_item('start_date', trigger, trigger_args)
+            self.__copy_item('end_date', trigger, trigger_args)
+            self.__copy_item('timezone', trigger, trigger_args)
+        elif trigger_type == 'cron':
+            self.__copy_item('year', trigger, trigger_args)
+            self.__copy_item('month', trigger, trigger_args)
+            self.__copy_item('day', trigger, trigger_args)
+            self.__copy_item('week', trigger, trigger_args)
+            self.__copy_item('day_of_week', trigger, trigger_args)
+            self.__copy_item('hour', trigger, trigger_args)
+            self.__copy_item('minute', trigger, trigger_args)
+            self.__copy_item('second', trigger, trigger_args)
+        elif trigger_type == 'date':
+            self.__copy_item('run_date', trigger, trigger_args)
+            self.__copy_item('timezone', trigger, trigger_args)
         else:
             raise Exception('Trigger %s is invalid.' % trigger)
 
-        self.__scheduler.add_job(call_func, trigger, func_args, func_kwargs, id, **trigger_args)
+        job = self.__scheduler.add_job(call_func, trigger_type, func_args, func_kwargs, id, **trigger_args)
+        job.func_ref = func_ref
+
+    @staticmethod
+    def __load_views(app):
+        app.add_url_rule('/jobs', 'get_jobs', get_jobs)
+        app.add_url_rule('/jobs/<job_id>', 'get_job', get_job)
+        app.add_url_rule('/jobs/<job_id>/run', 'run_job', run_job)
 
     @staticmethod
     def __copy_item(prop, src, dst):
